import React, { useState } from 'react';
import { useDispatch } from 'react-redux';
import { setCandies, setMaxWeight, knapsack } from './counterSlice';
import './counter.module.css';

const Counter = () => {
  const dispatch = useDispatch();
  const [candyCount, setCandyCount] = useState(0);
  const [candies, setCandiesState] = useState([]);
  const [maxWeight, setMaxWeightState] = useState(0);

  const handleCandyCountChange = (e) => {
    const count = parseInt(e.target.value, 10);
    setCandyCount(count);
    setCandiesState(Array(count).fill({ name: '',
      tastiness: 0,
      weight: 0 }));
  };

  const handleCandyChange = (index, field, value) => {
    const updatedCandies = candies.map((candy, i) =>
      i === index ? { ...candy,
        [field]: value } : candy,
    );
    setCandiesState(updatedCandies);
  };

  const handleSubmit = () => {
    dispatch(setCandies(candies));
    dispatch(setMaxWeight(maxWeight));
    dispatch(knapsack());
  };

  return (
    <div>
      <h2 classname="setUpStore">Set Up Candy Store</h2>
      <label classname="numberOfCandies">
        Number of candies:
        <input
          type="number"
          value={candyCount}
          onChange={(e) => handleCandyCountChange(e)}
        />
      </label>
      {candies.map((candy, index) =>
        <div key={index}>
          <label classname="candieName">
            Name:
            <input
              type="text"
              value={candy.name}
              onChange={(e) =>
                handleCandyChange(index, 'name', e.target.value)
              }
            />
          </label>
          <label classname="candieTastiness">
            Tastiness:
            <input
              type="number"
              value={candy.tastiness}
              onChange={(e) =>
                handleCandyChange(index, 'tastiness', parseInt(e.target.value, 10))
              }
            />
          </label>
          <label classname="candieWeight">
            Weight:
            <input
              type="number"
              value={candy.weight}
              onChange={(e) =>
                handleCandyChange(index, 'weight', parseInt(e.target.value, 10))
              }
            />
          </label>
        </div>,
      )}
      <label>
        Max Basket Weight:
        <input
          type="number"
          value={maxWeight}
          onChange={(e) => setMaxWeightState(parseInt(e.target.value, 10))}
        />
      </label>
      <button onClick={handleSubmit}>Calculate Knapsack</button>
    </div>
  );
};

export default Counter;
