import { createSlice } from '@reduxjs/toolkit';

const counterSlice = createSlice({
  name: 'counter',
  initialState: {
    candies: [],
    maxCandyWeight: 0,
    result: {
      totalTastiness: 0,
      quantities: [],
    },
  },
  reducers: {
    setCandies: (state, action) => {
      state.candies = action.payload;
    },
    setMaxWeight: (state, action) => {
      state.maxWeight = action.payload;
    },
    knapsack: (state) => {
      const { candies, maxWeight } = state;

      const dp = Array(maxWeight + 1).fill(0);
      const quantities = Array(maxWeight + 1)
        .fill(0)
        .map(() => Array(candies.length).fill(0));

      for (let w = 0; w <= maxWeight; w++) {
        for (let i = 0; i < candies.length; i++) {
          const { tastiness, weight } = candies[i];
          if (weight <= w) {
            const newTastiness = dp[w - weight] + tastiness;
            if (newTastiness > dp[w]) {
              dp[w] = newTastiness;
              quantities[w] = [...quantities[w - weight]];
              quantities[w][i] += 1;
            }
          }
        }
      }

      state.result.totalTastiness = dp[maxWeight];
      state.result.quantities = quantities[maxWeight];
    },
  },
});
export const { setCandies, setMaxWeight, knapsack } = counterSlice.actions;

export const selectCandies = (state) => state.counter.candies;
export const selectMaxWeight = (state) => state.counter.maxWeight;
export const selectResult = (state) => state.counter.result;

export default counterSlice.reducer;
